@@ -4,7 +4,7 @@ Provides simpler methods for :class:`~.cloud_wanderer.CloudWanderer` to call.
 """
 
 import logging
-from typing import TYPE_CHECKING, Iterator, List, Optional, cast
+from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, cast
 
 import botocore
 from boto3.resources.base import ServiceResource
@@ -78,6 +78,7 @@ class CloudWandererAWSInterface:
                 raise UnsupportedResourceTypeError(f"Resource type {urn.resource_type} doesn't support get_resource()")
             logger.info("Loading resource data.")
             resource.load()
+            resource.fetch_secondary_attributes()
 
         except botocore.exceptions.ClientError as ex:
             error_code = ex.response.get("ResponseMetadata", {}).get("HTTPStatusCode")
@@ -103,6 +104,7 @@ class CloudWandererAWSInterface:
                     resource.get_urn().resource_id,
                 )
                 for dependent_resource in resource.collection(resource_type=dependent_resource_type):
+                    dependent_resource.fetch_secondary_attributes()
                     logger.debug("Found %s", dependent_resource)
                     if not dependent_resource.meta.data and hasattr(dependent_resource, "load"):
                         dependent_resource.load()
@@ -110,16 +112,13 @@ class CloudWandererAWSInterface:
                     dependent_resource_urns.append(urn)
                     yield CloudWandererResource(
                         urn=urn,
-                        resource_data={
-                            **dependent_resource.normalized_raw_data,
-                            **dependent_resource.get_secondary_attributes_map(),
-                        },
+                        resource_data=dependent_resource.normalized_raw_data,
                         parent_urn=resource.get_urn(),
                         relationships=dependent_resource.relationships,
                     )
         yield CloudWandererResource(
             urn=resource.get_urn(),
-            resource_data={**resource.normalized_raw_data, **resource.get_secondary_attributes_map()},
+            resource_data=resource.normalized_raw_data,
             dependent_resource_urns=dependent_resource_urns,
             relationships=resource.relationships,
         )
@@ -129,6 +128,7 @@ class CloudWandererAWSInterface:
         service_name: str,
         resource_type: str,
         region: str,
+        filters: Dict[str, Any] = None,
         **kwargs,
     ) -> Iterator[CloudWandererResource]:
         """Return all resources of resource_type from Boto3.
@@ -137,6 +137,7 @@ class CloudWandererAWSInterface:
             service_name (str): The name of the service to get resource for (e.g. ``'ec2'``)
             resource_type (str): The type of resource to get resources of (e.g. ``'instance'``)
             region (str): The region to get resources of (e.g. ``'eu-west-1'``)
+            filters: A dictionary to be passed to the Boto3 method for filtering this resource.
             **kwargs: Additional keyword arguments will be passed down to the Boto3 client.
 
         Raises:
@@ -148,7 +149,10 @@ class CloudWandererAWSInterface:
         resource_map: ResourceMap = service.service_map.get_resource_map(resource_type)
 
         try:
-            for resource in service.collection(resource_type=resource_type, filters=resource_map.default_filters):
+            for resource in service.collection(
+                resource_type=resource_type, filters=filters or resource_map.default_filters
+            ):
+                resource.fetch_secondary_attributes()
                 dependent_resource_urns = []
                 for dependent_resource_type in resource.dependent_resource_types:
                     logger.info(
@@ -159,6 +163,7 @@ class CloudWandererAWSInterface:
                         resource.get_urn().resource_id,
                     )
                     for dependent_resource in resource.collection(resource_type=dependent_resource_type):
+                        dependent_resource.fetch_secondary_attributes()
                         logger.debug("Found %s", dependent_resource)
                         if not dependent_resource.meta.data and hasattr(dependent_resource, "load"):
                             dependent_resource.load()
@@ -166,16 +171,13 @@ class CloudWandererAWSInterface:
                         dependent_resource_urns.append(urn)
                         yield CloudWandererResource(
                             urn=urn,
-                            resource_data={
-                                **dependent_resource.normalized_raw_data,
-                                **dependent_resource.get_secondary_attributes_map(),
-                            },
+                            resource_data=dependent_resource.normalized_raw_data,
                             parent_urn=resource.get_urn(),
                             relationships=dependent_resource.relationships,
                         )
                 yield CloudWandererResource(
                     urn=resource.get_urn(),
-                    resource_data={**resource.normalized_raw_data, **resource.get_secondary_attributes_map()},
+                    resource_data=resource.normalized_raw_data,
                     dependent_resource_urns=dependent_resource_urns,
                     relationships=resource.relationships,
                 )
