@@ -1,7 +1,8 @@
 """Create the CloudWandererServiceResource objects that do the magic."""
 import logging
 import re
-from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, List, Optional, Type
+from collections import defaultdict
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Type
 
 import jmespath  # type: ignore
 from boto3.resources.base import ServiceResource
@@ -19,6 +20,7 @@ from ..urn import URN, PartialUrn
 from ..utils import snake_to_pascal
 from .boto3_helpers import _clean_boto3_metadata
 from .boto3_loaders import MergedServiceLoader, ServiceMap
+from .exceptions import SecondaryAttributesNotFetchedError
 
 if TYPE_CHECKING:
     from .session import CloudWandererBoto3Session
@@ -29,6 +31,20 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
+def _get_urn_components_from_string(regex_pattern, string) -> Dict[str, Any]:
+    result = re.match(regex_pattern, string)
+    if not result:
+        return {}
+    urn_components = defaultdict(list)
+    for arg_name, arg_value in result.groupdict().items():
+        if arg_name in ["cloud_name", "account_id", "region", "service", "resource_type"]:
+            urn_components[arg_name] = arg_value
+            continue
+        if arg_name.startswith("id_part_"):
+            urn_components["resource_id_parts"].append(arg_value)
+    return urn_components
+
+
 class CloudWandererResourceFactory(ResourceFactory):
     """Enriches functionality of boto3 resource objects with CloudWanderer specific methods."""
 
@@ -75,7 +91,7 @@ class CloudWandererResourceFactory(ResourceFactory):
             # This should only exist only exist on Resources, not on Services
             attrs["get_discovery_action_templates"] = self._create_get_discovery_action_templates()
             # attrs["get_dependent_resource"] = self._create_get_dependent_resource(service_context)
-            attrs["get_secondary_attributes_map"] = self._create_get_secondary_attributes_map()
+            attrs["secondary_attributes_map"] = self._create_secondary_attributes_map()
             if hasattr(original_class_definition, "load"):
                 attrs["load"] = self._create_load(original_class_definition=original_class_definition)
         else:
@@ -86,7 +102,7 @@ class CloudWandererResourceFactory(ResourceFactory):
         attrs["get_account_id"] = self._create_get_account_id()
         attrs["get_urn"] = self._create_get_urn()
         attrs["get_region"] = self._create_get_region()
-        attrs["get_secondary_attributes"] = self._create_get_secondary_attributes()
+        attrs["fetch_secondary_attributes"] = self._create_fetch_secondary_attributes()
 
     def _create_load(self, original_class_definition: Any) -> Callable:
         if not hasattr(original_class_definition, "load"):
@@ -105,7 +121,7 @@ class CloudWandererResourceFactory(ResourceFactory):
                     f"{self.service_name} {self.resource_type} does not have a load definition in its resources-1.json"
                 )
 
-            has_non_empty_values = any(list([x for x in identifiers.values()]))
+            has_non_empty_values = any(x for x in identifiers.values())
             if not has_non_empty_values:
                 logger.debug(
                     "Load is a noop on this %s %s because we are an empty_resource=True resource",
@@ -234,39 +250,64 @@ class CloudWandererResourceFactory(ResourceFactory):
     def _create_get_urn(self) -> Callable:
         def get_urn(self) -> URN:
             id_parts = [getattr(self, identifier) for identifier in self.meta.identifiers]
-            return URN(
-                account_id=self.get_account_id(),
-                region=self.get_region(),
-                service=self.service_name,
-                resource_type=self.resource_type,
-                resource_id_parts=id_parts,
-            )
+            urn_args = {
+                "account_id": self.get_account_id(),
+                "region": self.get_region(),
+                "service": self.service_name,
+                "resource_type": self.resource_type,
+                "resource_id_parts": id_parts,
+            }
+            for urn_override in self.resource_map.urn_overrides:
+                urn_args.update(
+                    _get_urn_components_from_string(
+                        urn_override.regex_pattern, jmespath.search(urn_override.path, self.meta.data)
+                    )
+                )
+            return URN(**urn_args)
 
         return get_urn
 
-    def _create_get_secondary_attributes_map(self) -> Callable[..., Dict[str, Any]]:
-        def get_secondary_attributes_map(self) -> Dict[str, Any]:
-            """Return a dictionary representation of this resource's secondary attributes."""
+    def _create_secondary_attributes_map(self) -> property:
+        def secondary_attributes_map(self) -> Dict[str, Any]:
+            """Return a dictionary representation of this resource's secondary attributes.
+
+            Raises:
+                SecondaryAttributesNotFetchedError: When the secondary attributes haven't been fetched yet.
+            """
+            if not self._secondary_attributes_fetched:
+                raise SecondaryAttributesNotFetchedError(
+                    "Secondary attributes have not yet been fetched for this "
+                    "resource. Call fetch_secondary_attributes first."
+                )
             result = {}
-            for secondary_attribute in self.get_secondary_attributes():
-                logger.debug("Getting secondary attribute: %s", secondary_attribute)
+            for secondary_attribute in self._secondary_attributes:
                 for attribute_map in secondary_attribute.resource_map.secondary_attribute_maps:
                     result[attribute_map.destination_name] = jmespath.search(
                         attribute_map.source_path, secondary_attribute.normalized_raw_data
                     )
             return result
 
-        return get_secondary_attributes_map
+        return property(secondary_attributes_map)
+
+    def _create_fetch_secondary_attributes(self) -> Callable:
+        def fetch_secondary_attributes(self) -> None:
+            self._secondary_attributes = []
 
-    def _create_get_secondary_attributes(self) -> Callable:
-        def get_secondary_attributes(self) -> Generator["CloudWandererServiceResource", None, None]:
             for secondary_attribute_name in self.secondary_attribute_names:
+                logger.info(
+                    "Getting %s secondary attributes from %s for %s",
+                    secondary_attribute_name,
+                    self.get_region(),
+                    self.get_urn().resource_id,
+                )
                 getter = getattr(self, snake_to_pascal(secondary_attribute_name))
                 secondary_attribute_resource = getter()
                 secondary_attribute_resource.load()
-                yield secondary_attribute_resource
+                secondary_attribute_resource.fetch_secondary_attributes()
+                self._secondary_attributes.append(secondary_attribute_resource)
+            self._secondary_attributes_fetched = True
 
-        return get_secondary_attributes
+        return fetch_secondary_attributes
 
     def _create_secondary_attribute_names(self) -> property:
         def secondary_attribute_names(self) -> List[str]:
@@ -308,6 +349,7 @@ class CloudWandererResourceFactory(ResourceFactory):
             """Return the raw data dictionary for this resource, ensuring that all possible keys are present."""
             result = {attribute: None for attribute in self.shape.members.keys()}
             result.update(self.meta.data or {})
+            result.update(self.secondary_attributes_map)
             return _clean_boto3_metadata(result)
 
         return property(normalized_raw_data)
@@ -357,10 +399,7 @@ class CloudWandererResourceFactory(ResourceFactory):
             for relationship_specification in self.resource_map.relationships:
                 base_paths_raw = jmespath.search(
                     relationship_specification.base_path,
-                    {
-                        **self.normalized_raw_data,
-                        **self.get_secondary_attributes_map(),
-                    },
+                    self.normalized_raw_data,
                 )
                 base_paths = [base_paths_raw] if not isinstance(base_paths_raw, list) else base_paths_raw
                 for base_path in base_paths:
@@ -389,15 +428,11 @@ class CloudWandererResourceFactory(ResourceFactory):
                         if not id_part.regex_pattern:
                             urn_args["resource_id_parts"].append(id_raw)
                             continue
-                        result = re.match(id_part.regex_pattern, id_raw)
-                        if not result:
+                        regex_results = _get_urn_components_from_string(id_part.regex_pattern, id_raw)
+                        if not regex_results:
                             continue
-                        for arg_name, arg_value in result.groupdict().items():
-                            if arg_name in ["cloud_name", "account_id", "region", "service", "resource_type"]:
-                                urn_args[arg_name] = arg_value
-                                continue
-                            if arg_name.startswith("id_part_"):
-                                urn_args["resource_id_parts"].append(arg_value)
+                        urn_args.update(regex_results)
+
                     if not urn_args["resource_id_parts"]:
                         continue
                     relationships.append(
@@ -438,3 +473,4 @@ class CloudWandererResourceFactory(ResourceFactory):
             attrs["shape"] = self._create_shape()
             attrs["relationships"] = self._create_relationships()
             attrs["is_dependent_resource"] = self._create_is_dependent_resource()
+            attrs["_secondary_attributes_fetched"] = False
