@@ -1,6 +1,7 @@
 """Storage Connectors for CloudWanderer."""
 from .base_connector import BaseStorageConnector
 from .dynamodb import DynamoDbConnector
+from .gremlin import GremlinStorageConnector
 from .memory import MemoryStorageConnector
 
-__all__ = ["DynamoDbConnector", "MemoryStorageConnector", "BaseStorageConnector"]
+__all__ = ["DynamoDbConnector", "MemoryStorageConnector", "BaseStorageConnector", "GremlinStorageConnector"]
